// General-purpose Photovoltaic Device Model gpvdm.com - a drift diffusion
// base/Shockley-Read-Hall model for 1st, 2nd and 3rd generation solarcells.
// The model can simulate OLEDs, Perovskite cells, and OFETs.
// 
// Copyright (C) 2008-2020 Roderick C. I. MacKenzie
// All rights reserved.
// 
// https://www.gpvdm.com
// r.c.i.mackenzie at googlemail.com
// 
// 
// Redistribution and use in source and binary forms, with or without
// modification, are permitted provided that the following conditions are met:
//     * Redistributions of source code must retain the above copyright
//       notice, this list of conditions and the following disclaimer.
//     * Redistributions in binary form must reproduce the above copyright
//       notice, this list of conditions and the following disclaimer in the
//       documentation and/or other materials provided with the distribution.
//     * Neither the name of the GPVDM nor the
//       names of its contributors may be used to endorse or promote products
//       derived from this software without specific prior written permission.
// 
// THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND
// ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
// WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
// DISCLAIMED. IN NO EVENT SHALL Roderick C. I. MacKenzie BE LIABLE FOR ANY
// DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
// (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
// LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
// ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
// (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
// SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.


#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <math.h>
#include <errno.h>
#include <util.h>
#include <unistd.h>
#include <dlfcn.h>
#include <sys/types.h>
#include <dirent.h>

#include "light_interface.h"
#include "true_false.h"
#include "device.h"
#include "dump_ctrl.h"
#include "config.h"

static int unused __attribute__((unused));

void light_init(struct light *in,struct device *cell)
{
char *error;
char mode[20];
char lib_path[200];
char name[200];
double ver;
FILE *config;
config=fopena(cell->outputpath,"light.inp","r");
if (config==NULL)
{
	ewe("light.inp not found\n");
}

config_read_line_to_double(&(cell->Psun),config,"#Psun");

config_read_line_to_string(mode,config,"#light_model");

config_read_line_to_double(&(in->electron_eff),config,"#electron_eff");

config_read_line_to_double(&(in->hole_eff),config,"#hole_eff");

config_read_line_to_double(&(in->Dphotoneff),config,"#Dphotoneff");

config_read_line_to_double(&(in->ND),config,"#NDfilter");

config_read_line_to_double(&(ver),config,"#ver");

if (ver!=1.2)
{
	ewe("File compatability problem %s\n","light.inp");
}

unused=fscanf(config,"%s",name);
if (strcmp("#end",name)!=0)
{
ewe("Problem with light.inp file last item read %s!\n",name);
}
fclose(config);

sprintf(lib_path,"./light/%s.so",mode);

if( access( lib_path, F_OK ) == -1 )
{
	sprintf(lib_path,"/usr/lib64/opvdm/%s.so",mode);
}


	in->lib_handle = dlopen(lib_path, RTLD_LAZY);
	if (!in->lib_handle)
	{
		fprintf(stderr, "%s\n", dlerror());
		exit(0);
	}

	in->fn_init = dlsym(in->lib_handle, "light_init");
	if ((error = dlerror()) != NULL)
	{
		fprintf(stderr, "%s\n", error);
		exit(0);
	}

	in->fn_load_config = dlsym(in->lib_handle, "light_load_config");
	if ((error = dlerror()) != NULL)
	{
		fprintf(stderr, "%s\n", error);
		exit(0);
	}

	in->fn_solve_and_update = dlsym(in->lib_handle, "light_solve_and_update");
	if ((error = dlerror()) != NULL)
	{
		fprintf(stderr, "%s\n", error);
		exit(0);
	}

	in->fn_free = dlsym(in->lib_handle, "light_free");
	if ((error = dlerror()) != NULL)
	{
		fprintf(stderr, "%s\n", error);
		exit(0);
	}

	in->fn_cal_photon_density = dlsym(in->lib_handle, "light_cal_photon_density");
	if ((error = dlerror()) != NULL)
	{
		fprintf(stderr, "%s\n", error);
		exit(0);
	}

	in->fn_solve_lam_slice = dlsym(in->lib_handle, "light_solve_lam_slice");
	if ((error = dlerror()) != NULL)
	{
		fprintf(stderr, "%s\n", error);
		exit(0);
	}

	(*in->fn_init)();
}

void light_solve_and_update(struct device *cell,struct light *in,double Psun_in,double laser_eff_in)
{
(*in->fn_solve_and_update)(cell,in,Psun_in,laser_eff_in);
}

void light_load_config(struct light *in,char *output_path)
{
printf("Loading config\n");
(*in->fn_load_config)(in,output_path);
}

double light_cal_photon_density(struct light *in)
{
return (*in->fn_cal_photon_density)(in);
}

int light_solve_lam_slice(struct light *in,int lam)
{
return (*in->fn_solve_lam_slice)(in,lam);
}


void light_free(struct light *in)
{
dlclose(in->lib_handle);
}
