// General-purpose Photovoltaic Device Model gpvdm.com - a drift diffusion
// base/Shockley-Read-Hall model for 1st, 2nd and 3rd generation solarcells.
// The model can simulate OLEDs, Perovskite cells, and OFETs.
// 
// Copyright (C) 2008-2020 Roderick C. I. MacKenzie
// All rights reserved.
// 
// https://www.gpvdm.com
// r.c.i.mackenzie at googlemail.com
// 
// 
// Redistribution and use in source and binary forms, with or without
// modification, are permitted provided that the following conditions are met:
//     * Redistributions of source code must retain the above copyright
//       notice, this list of conditions and the following disclaimer.
//     * Redistributions in binary form must reproduce the above copyright
//       notice, this list of conditions and the following disclaimer in the
//       documentation and/or other materials provided with the distribution.
//     * Neither the name of the GPVDM nor the
//       names of its contributors may be used to endorse or promote products
//       derived from this software without specific prior written permission.
// 
// THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND
// ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
// WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
// DISCLAIMED. IN NO EVENT SHALL Roderick C. I. MacKenzie BE LIABLE FOR ANY
// DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
// (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
// LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
// ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
// (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
// SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.


#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <math.h>
#include "../../sim.h"
#include "../../ntricks.h"
#include "../../render1d.h"
#include "tof.h"

static int unused __attribute__((unused));

struct tof tof_config;

void sim_tof(struct device *in)
{
gui_start();
double V=0.0;
int ittr=0;
int step=0;

tof_load_config(&tof_config,in);
printf("Running ToF. %e\n",tof_config.tof_voltage);

in->go_time=FALSE;

dump_dynamic_init(in);

in->dt=tof_config.tof_dt;

light_solve_and_update(in,&(in->mylight),in->Psun,0.0);

if (fabs(tof_config.tof_voltage-in->Vapplied)>0.2)
{
	ramp_externalv(in,0.0,tof_config.tof_voltage);
}


sim_externalv(in,tof_config.tof_voltage);

tof_config.tof_power_suply=tof_config.tof_voltage+get_equiv_I(in)*tof_config.tof_Rshort;

time_init(in);
device_timestep(in);



in->time=-tof_config.start_steps*in->dt;

printf("%le\n",tof_config.tof_power_suply);

struct istruct tofout;
inter_init(&tofout);

in->go_time=TRUE;
V=in->Vapplied;

//int dump_count=0;

carrier_count_reset(in);
render1d_reset();



do
{
	if (step<tof_config.start_steps)
	{
		light_solve_and_update(in,&(in->mylight),in->Psun,0.0);
		reset_npinit(in);
	}else
	if ((step>=tof_config.start_steps)&&(in->time<(tof_config.toflaseroff)))
	{
		light_solve_and_update(in,&(in->mylight),in->Psun,tof_config.toflaserpower);
		stop_start(in);
	}else
	{
		light_solve_and_update(in,&(in->mylight),in->Psun,0.0);
		stop_start(in);
	}

	in->Vapplied=V;

	double i0=newton_sim_tof_all(in,TRUE);

	if (get_dump_status(dump_print_text)==TRUE)
	{
	printf("tof time=%e step=%d %.1e ",in->time,step,in->last_error);
	printf("V=%lf current = %e mA (%e A/m^2)\n",V,i0/1e-3,get_J(in));
	}

	ittr++;

	plot_now(in,"tof.plot");
	dump_dynamic_add_data(in);

	slice_check(in);

	if (in->time>0.0) in->dt*=tof_config.tofdtmull;

	inter_append(&tofout, in->time, fabs(i0));

	device_timestep(in);
	step++;

	if (in->time>tof_config.tofstop) break;

	if (in->stop==TRUE)
	{
		break;

	}

	stop_start(in);

}while(1);


in->go_time=FALSE;

char outpath[200];

inter_sub_double(&tofout,tofout.data[tofout.len-1]);

sprintf(outpath,"%s%s",in->outputpath,"tof_i.dat");
inter_save(&tofout,outpath);

sprintf(outpath,"%s%s",in->outputpath,"tof_dos.dat");
int i;
inter_chop(&tofout,1e-9, 100.0);
for (i=0;i<tofout.len;i++)
{
	tofout.data[i]*=tofout.x[i];
	tofout.x[i]=((300.0*kb/q)*log(1e7*tofout.x[i]));
}

inter_save(&tofout,outpath);

inter_free(&tofout);

dump_dynamic_save(in);
dump_dynamic_free(in);
gui_stop();
}

void tof_load_config(struct tof *in,struct device *dev)
{
double ver=0.0;
char name[200];
FILE *config=fopena(dev->outputpath,"tof.inp","r");
if (config==NULL)
{
ewe("tof.inp file not found\n");
}

unused=fscanf(config,"%s",name);
unused=fscanf(config,"%le",&(in->tofstop));
unused=fscanf(config,"%s",name);
unused=fscanf(config,"%le",&(in->toflaseroff));
unused=fscanf(config,"%s",name);
unused=fscanf(config,"%le",&(in->tof_dt));
unused=fscanf(config,"%s",name);
unused=fscanf(config,"%le",&(in->tofdtmull));
unused=fscanf(config,"%s",name);
unused=fscanf(config,"%le",&(in->toflaserpower));
unused=fscanf(config,"%s",name);
unused=fscanf(config,"%le",&(in->start_steps));
unused=fscanf(config,"%s",name);
unused=fscanf(config,"%le",&(in->tof_voltage));
unused=fscanf(config,"%s",name);
unused=fscanf(config,"%le",&(in->tof_Rshort));
unused=fscanf(config,"%s",name);
unused=fscanf(config,"%lf",&(ver));
if (ver!=1.1)
{
ewe("File compatability problem %s\n","tof.inp");
}

unused=fscanf(config,"%s",name);
if (strcmp("#end",name)!=0)
{
ewe("Problem with tof.inp file last item read %s!\n",name);
}
fclose(config);
//</clean>
}
